function varargout = llrfittedqi_exp(mode, cfg)
% Handle experiment in batch run, for this type of algorithm

% TODO test and apply cleanup

if nargin < 2, cfg = struct; end;

switch mode,
    % provide short ID
    case 'shortid',
        varargout = {'llrfqi'};
    
    % provide configuration parameters
    case 'params',
        p = struct;
        % main parameters
        p.problem = [];
        p.gamma = [];
        p.U.id = 'U'; p.U.type = 'cell';
        p.knn.id = 'knn'; p.knn.shortid  = 'k'; p.knn.label = 'k_{NN}';
        p.S.id = 'S'; p.S.type = 'vector';
        p.wfun.id = 'wfun'; p.wfun.shortid = 'w'; p.wfun.label = 'weighting function'; p.wfun.type = 'char';
        p.extrap = [];
        p.samplesmethod.path = 'samples.method'; p.samplesmethod.shortid = '';
        p.samplesN.path = 'samples.N'; p.samplesN.shortid = 'N';
        p.samplesgridfun.path = 'samples.gridfun'; p.samplesgridfun.shortid = 'fun';
        p.samplesxgrids.path = 'samples.xgrids'; 
        p.samplesxgrids.type = 'vector'; p.samplesxgrids.shortid = 'xgr';   
            % note we assume we won't explicitly use cell arrays of points, 
            % thus allowing for multiple settings using cell array of vectors
        p.samplesugrids.path = 'samples.ugrids';
        p.samplesugrids.type = 'array'; p.samplesugrids.shortid = 'ugr';
        % remaining, "trivial" parameters
        p = addtrivialparams(p, 'maxiter', 'eps', 'maxeps', 'checkosc', 'maxper', 'nper', 'epsper', ...
            'datadir', 'verb', 'iterdisp', 'itersave', 'sampledisp', 'storehistory', 'cleanuplevel');
        varargout = {p};
        
    % provide output variables
    case 'outputs',
        o = struct;
        o.score.id = 'score';
        o.score.label = 'score';
        o.score.params = {'gamma', 'X0', 'mc_eps', 'mc_N'};
        
        o.exectime.id = 'exectime';
        o.exectime.label = 'execution time';
        o.exectime.units = 's';
        o.exectime.scale = 'log';

        o.iter.id = 'iter';
        o.iter.label = 'number of iterations';
        
        varargout = {o};
        
    % process options
    case {'run', 'sol'},
        cfg.(mode) = 1;
        llrfittedqi(cfg);
        
    % compute execution time
    case 'exectime',
        load(cfg.datafile, 'timestat');
        varargout = {timestat.init + timestat.run};

    % find # iterations to completion
    case 'iter',
        load(cfg.datafile, 'k');
        varargout = {k-1};

    % evaluate score
    case 'score',
        load(cfg.datafile, 'model', 'approx', 'Qs');
        mccfg.gamma = cfg.gamma;
        mccfg.mc_eps = cfg.mc_eps;
        mccfg.mc_N = cfg.mc_N;
        % if X0 is string, call problem function to get actual states
        % note we are using cfg.problem, not model.problem
        if ischar(cfg.X0), cfg.X0 = feval(cfg.problem, 'X0', cfg.X0); end;
        score = mc_approx(model, approx, Qs, cfg.X0, mccfg);
        varargout = {score};

    case 'cleanup',
        cfgc = cfg; clear cfg;      % to prevent overlap with data variable name
        % if ~isfield(cfgc, 'cleanuplevel'), cfgc.cleanuplevel = 1; end;
        cfgc.datafile = which([cfgc.datafile '.mat']);     % including full path
        
        % get list of all variable names
        varnames = who('-file', cfgc.datafile);
        % decide which to remove
        % level 1: large auxiliary variables -- resume theoretically possible (not implemented yet)
        remove = {};        
        if cfgc.cleanuplevel >= 1, remove = {remove{:}, 'INDLLR', 'WLLR', 'XLLRpinv', 'dist', 'ind'}; end;
        % level 2: also Q-sample history
        if cfgc.cleanuplevel >= 2, remove = {remove{:}, 'Qsh'}; end;
        % level 3: also samples, iteration helpers etc -- resume no longer possible except by
        % recovering data from approximator
        if cfgc.cleanuplevel >= 3, remove = {remove{:}, 'Qsold', 'Xs', 'Us', 'Xps', 'Rs', 'Is', 'xllr'}; end;
        % remove from list
        varnames = setdiff(varnames, remove);
        
        % load ONLY remaining variables (should save some time)
        load(cfgc.datafile, varnames{:});
        % test if cleanup already performed
        if isfield(cfg, 'cleanedup') && cfg.cleanedup >= cfgc.cleanuplevel,
            disp(sprintf('WARNING! Datafile [%s] was already level-%d cleaned-up. Skipping.', ...
                cfgc.datafile, cfgc.cleanuplevel));
            return;
        end;
        % additional cleanup within variables that are not removed
        if cfgc.cleanuplevel >= 3,
            cfg.samples = rmfield(cfg.samples, {'X', 'U', 'Xp', 'R'});
            cfg.samples.cleanedup = cfgc.cleanuplevel;
        end;
        % flag that the file was cleaned up
        cfg.cleanedup = cfgc.cleanuplevel;
        % resave variables, overwriting old file
        save(cfgc.datafile, varnames{:});
        
    case 'cleanupdir',   
        % cleanup all files in the current directory 
        % note this mode is not called by process/evalbatch, but should be called top-level
        files = dir;
        cfgc = cfg;
        for i = 1:length(files),
            % only consider mat files
            extpos = strfind(files(i).name, '.mat');
            if extpos,
                df = files(i).name(1:extpos-1);
                % heuristic check whether this is indeed a LLR Fitted QI experiment and not something else
                load(df, 'cfg');
                llrexp = isfield(cfg, 'knnmethod') && isfield(cfg, 'wfun') && ~isfield(cfg, 'Nstau');
                % last condition is to differentiate from interlaced, illrfittedqi
                if llrexp, 
                    disp(sprintf('Level %d cleanup of datafile [%s].', cfgc.cleanuplevel, df));
                    cfgc.datafile = df;
                    llrfittedqi_exp('cleanup', cfgc); 
                else
                    disp(sprintf('Datafile [%s] does not contain a LLRFittedQI experiment. Skipping.', df));
                end;
            end;
        end;
        
    otherwise,  % return empty
        varargout = {[]};
end;

