function varargout = ooconsensus_exp(mode, cfg)
% Handle experiment in batch run, for this type of algorithm

if nargin < 2, cfg = struct; end;

switch mode,
    % provide short ID
    case 'shortid',
        varargout = {'oocons'};

    % provide configuration parameters
    case 'params',
        p = struct;
        % main parameters
        p.problem = [];
        p.X.path = 'X'; p.X.type = 'vector';
        p.trials = []; p.maxsteps = [];
        p.reset.path = 'reset'; p.reset.type = 'array'; p.reset.shortid = 'res';
        p.alpha.path = 'alpha'; p.alpha.shortid = 'alpha'; p.alpha.label = '\alpha';
        p.P.path = 'P'; p.P.type = 'array';
        p.K.path = 'K'; p.K.shortid = 'K';
        % optimization parameters
        p.optparamsmethod.path = 'optparams.method'; p.optparamsmethod.shortid = 'alg'; p.optparamsmethod.label = 'algorithm'; % p.optparamsn.scale = 'log';
        p.optparamsn.path = 'optparams.n'; p.optparamsn.shortid = 'n'; p.optparamsn.label = 'budget'; % p.optparamsn.scale = 'log';
        p.optparamsmaxu.path = 'optparams.maxu'; p.optparamsmaxu.type = 'array'; 
        p.optparamdiam.path = 'optparams.diam'; 
        p.optparamdmax.path = 'optparams.dmax'; 
        % remaining, "trivial" parameters
        p = addtrivialparams(p, 'model_params', 'reset', 'datadir', 'verb', 'trialdisp', 'repeat');
        varargout = {p};
        
    % provide output variables
    case 'outputs',
        o = struct;

        % disagreement between agent velocities
        o.disagr.id = 'disagr';
        o.disagr.label = 'disagreement';
        o.disagr.params = {'steps'};

        % total execution time
        o.exectime.id = 'exectime';
        o.exectime.label = 'execution time';
        o.exectime.units = 's';
        o.exectime.scale = 'log';

        varargout = {o};
        
    % process options
    case {'run'},
        cfg.(mode) = 1;
        ooconsensus(cfg);
        
    % compute execution time
    case 'exectime',
        load(cfg.datafile, 'timestat');
        varargout = {timestat.run};

    % evaluate disagreement on the entire state
    case 'disagr',
        % load experiment config
        loadwithprefix(cfg.datafile, 'e_', {'cfg', 'X', 'model'});
        if isempty(cfg.steps),  steps = 1:e_cfg.maxsteps+1;
        else                    steps = cfg.steps;
        end;
        % compute disagreement vector for each trial and step
        disagr = zeros(length(steps), e_cfg.trials);
        for trial = 1:e_cfg.trials,
            x = reshape(e_X(:, :, trial), e_model.p(1), e_model.nagents, e_cfg.maxsteps+1);
            % compute sum of distances explicitly using FORs, for clarity
            for is = 1:length(steps),
                s = steps(is);
                for i = 1:e_model.nagents,
                    for j = i+1:e_model.nagents,
                        disagr(is, trial) = disagr(is, trial) + sqrt(sum((x(:, i, s) - x(:, j, s)) .^ 2));
                    end;
                end;
            end;
        end;
        varargout = {mean(disagr, 1), disagr};
        
    otherwise,  % return empty
        varargout = {[]};
end;

end     % main function genmpc_exp
