function figh = plot2dtriang(cfg)
% This function is obsolete, use plotmfs instead

CFG.plottarget = 'screen';   % should be latex for exporting FIG and EPS files
CFG.savedir = '';
CFG.onedimfig = 'triang_1d';    % file names
CFG.twodimfig = 'triang_2d';
CFG.onedimfigsize = [350 230];
CFG.twodimfigsize = CFG.onedimfigsize + [50 10];
CFG.C1 = [-2 0 2];
CFG.C2 = [-2 0 2];
CFG.plot = [1 2];
CFG.flat2d = 0;
CFG.psfraglabels = 1;

if nargin < 1, cfg = struct; end;
if ischar(cfg),  cfg = str2cfg(cfg, fieldnames(CFG)); end;
cfg = checkparams(cfg, CFG);
cfg.grayscale = grayscalefromconfig(cfg);

% init
C1 = cfg.C1; C2 = cfg.C2;
mfs1 = gen_mfs(C1); mfs2 = gen_mfs(C2);
C1x = [C1(1) C1 C1(end)];
C2x = [C2(1) C2 C2(end)];

% grayscale styles
gs.lsty = {{'k-', 'LineWidth', 2}, {'k--', 'LineWidth', 2}, {'k-.', 'LineWidth', 2}};
% color styles
cs = gs;
% set style
if cfg.grayscale, sty = gs;  else sty = cs; end;

% readable labels
latexprop = {'Interpreter', 'LaTeX', 'FontSize',13};
rl.x = 'x'; rl.x1 = 'x_1'; rl.x2 = 'x_2'; rl.mu1 = '\mu(x_1)'; rl.mu2 = '\mu(x_1, x_2)';
psl.x1 = 'X1'; psl.x1 = 'X1'; psl.x2 = 'X2'; psl.mu1 = 'MU1'; psl.mu2 = 'MU2';
latl.x = 'x'; latl.x1 = 'x_1'; latl.x2 = 'x_2'; latl.mu1 = '\mu(x_1)'; latl.mu2 = '\mu(x)';
% latl.x = {'$x$', latexprop{:}}; latl.x1 = {'$x_1$', latexprop{:}}; latl.x2 = {'$x_2$', latexprop{:}}; 
% latl.mu1 = {'$\varphi(x_1)$', latexprop{:}};
% latl.mu2 = {'$\varphi(x)$', latexprop{:}};
if strcmp(cfg.plottarget, 'latex'),
    if cfg.psfraglabels,    labels = psl; 
    else                    labels = latl;
    end;
else    labels = rl;
end;
lfn = fieldnames(labels);
for i = 1:length(lfn), 
    if ~iscell(labels.(lfn{i})), labels.(lfn{i}) = {labels.(lfn{i})}; end; 
end;

figh = [];

% -------------------------
% Figure 1: 1-D MFs
if any(cfg.plot == 1),
    figh(end+1) = figurex(cfg.onedimfigsize);
    for i = 1:length(C1),
        plot(C1x(i:i+2), [0 1 0], sty.lsty{i}{:}); hold on;
    end;
    geps  = 0.005;
%     set(gca, 'XLim', [C1(1) C1(end)] + [-geps geps]);    
    xlabel(labels.x{:}); ylabel(labels.mu1{:});
    saveplot(figh(end), [cfg.savedir cfg.onedimfig], cfg.plottarget);
end;


% -------------------------
% Figure 2: 2-D composition
if any(cfg.plot == 2),
    figh(end+1) = figurex(cfg.twodimfigsize);
    Np = 14;
    if cfg.flat2d,  % gray faces, black outlines
        eint = zeros(3, 3);
        fint = 0.75 + zeros(3, 3);
    else
        eint = [0, .8, .3;
               .6, 0, .6; 
               .3, .8, 0];
        fint = [.8, .5, .8; 
                .8, .3, .8;
                .8, .5, .8];
    end;
    for i = 1:length(C1),
        for j =1:length(C2),
            % 2-D domain of composite MF i,j
            d = [C1x(i) : (C1x(i+2)-C1x(i))/Np : C1x(i+2)
                 C2x(j) : (C2x(j+2)-C2x(j))/Np : C2x(j+2)];
            mu = d*0;
            for i1 = 1:size(d, 2),
                m1 = mdegs(d(1, i1), mfs1); m1 = m1(i);
                for j1 = 1:size(d, 2),
                    m2 = mdegs(d(2, j1), mfs2); m2 = m2(j);
                    mu(i1, j1) = m1 * m2;
                end;
            end;
            mesh(d(1, :), d(2, :), mu', 'EdgeColor', eint(i,j)*[1 1 1], ...
                'FaceColor', fint(i,j)*[1 1 1], 'FaceAlpha', 1, 'EdgeAlpha', 1); hold on;
        end;
    end;
    
    set(gca, 'view', [-41 38]);

    % draw edges
    geps = 0.005;
    for i = 1:length(C1),
        if cfg.flat2d,      % use same style for all MFs
            plot3(C1x(i:i+2), C2(1) + [0 0 0] - geps, [0 1 0], sty.lsty{1}{:}); hold on;
        else
            plot3(C1x(i:i+2), C2(1) + [0 0 0] - geps, [0 1 0], sty.lsty{i}{:}); hold on;
        end;
    end;
    for i = 1:length(C1),
        if cfg.flat2d,      % use same style for all MFs
            plot3(C1(1) + [0 0 0] - geps, C2x(i:i+2), [0 1 0], sty.lsty{1}{:}); hold on;
        else
            plot3(C1(1) + [0 0 0] - geps, C2x(i:i+2), [0 1 0], sty.lsty{i}{:}); hold on;
        end;
    end;
    geps  = 0.007;
    set(gca, 'XLim', [C1(1) C1(end)] + [-geps geps]);
    set(gca, 'YLim', [C2(1) C2(end)] + [-geps geps]);
    xlabel(labels.x1{:}); ylabel(labels.x2{:}); zlabel(labels.mu2{:});
    saveplot(figh(end), [cfg.savedir cfg.twodimfig], cfg.plottarget);
end;



% END plot2dtriang