% Test discrete-time flocking with or without minimal robust subgraph condition satisfied
clc; clear all; close all;

%% Large config: 9 agents

mcfg = struct;
mcfg.type = 'doubleint';
mcfg.nagents = 9;
mcfg.dim = 2;
mcfg.range = 5;
mcfg.flock_dist = 5;
mcfg.flock_maxdvel = 5;
% mcfg.flock_alpha = .5;
% mcfg.flock_beta = .5;
mcfg.Ts = 1;

m = malinear_problem('model', mcfg);

gridstep = mcfg.range * .6;
 % setup grid structure in 2D
gr = zeros(2, 3, 3); 
gr(1, :, :) = [-1 0 1; -1 0 1; -1 0 1];
gr(2, :, :) = [1 1 1; 0 0 0; -1 -1 -1];

% initial position (always the same)
p0 = gr .* gridstep;
% velocities don't matter for robust graph, set them to 0
x = [p0; 0.*p0];
[Grob, rhorob, lambda2rob] = malinear_problem('minrobgraph', m, x(:));

% maximal norm of velocity disagreement vector given by theory
vrob = lambda2rob * rhorob / sqrt(2) / sqrt(sum(abs(gr(:))));
% some velocities violating condition to various extents
v0xy = [1 1.05 1.5] * vrob;  

% compose initial states
n0 = length(v0xy);
x0 = cell(n0, 1);
for i = 1:n0,
    x = [p0; gr .* v0xy(i)];
    x0{i} = x(:); 
end;


%% Basic control algorithm
cfg = struct;
cfg.problem = 'malinear_problem';
cfg.model_params = {mcfg};
cfg.planalg = 'masimplecontrol';
cfg.planparams.type = 'flock';
% 12 July 2012: gain 1/n < 1/max_degree to ensure convergence 
% (the previous gain was the default 1/max degrees + 1 but this required centralized knowledge)
cfg.planparams.alpha = 1/mcfg.nagents; 
cfg.visualize = 2;
cfg.viscfg.pause = .01;
cfg.datadir = pwd;
cfg.maxsteps = 40;

%% run in case 1
cfg.reset = x0{1}; cfg.run = 1; cfg.datafile = 'classical_malin_2d_vrob'; magenmpc(cfg);
%% run in case 2
cfg.reset = x0{2}; cfg.run = 1; cfg.datafile = 'classical_malin_2d_105vrob'; magenmpc(cfg);
%% run in case 3
cfg.reset = x0{3}; cfg.run = 1; cfg.datafile = 'classical_malin_2d_15vrob'; magenmpc(cfg);

%% Optimistic planning with sequential communication
cfg = struct;
cfg.problem = 'malinear_problem';
cfg.model_params = {mcfg};
cfg.comm = 'seq';
cfg.planalg = 'maopd_predictneighbors';
Ugrid = [-.5 -.1 0 .1 .5];
cfg.planparams.U = repmat({{Ugrid, Ugrid}}, 1, mcfg.nagents);
cfg.planparams.n = 200;
% cfg.planparams.gametype = 'max';
cfg.planparams.rob = 0;
% cfg.planparams.maxpredlength = 1;
cfg.planparams.verb = 2;
cfg.visualize = 2;
cfg.viscfg.pause = .01;
cfg.datadir = pwd;
cfg.maxsteps = 40;
cfg.verb = 3;
cfg.saveplandata = 0;

%% run in case 1
cfg.reset = x0{1}; cfg.run = 1; cfg.datafile = 'magenmpc_seq_malin_2d_vrob'; magenmpc(cfg);
%% run in case 2
cfg.reset = x0{2}; cfg.run = 1; cfg.datafile = 'magenmpc_seq_malin_2d_105vrob'; magenmpc(cfg);
%% run in case 3
cfg.reset = x0{3}; cfg.run = 1; cfg.datafile = 'magenmpc_seq_malin_2d_15vrob'; magenmpc(cfg);

%% Old code for previous-plans communicated

% %% Optimistic planning with previous-action predictions
% cfg = struct;
% cfg.problem = 'malinear_problem';
% cfg.model_params = {mcfg};
% cfg.comm = 'prev';
% cfg.planalg = 'maopd_predictneighbors';
% Ugrid = [-.5 -.1 0 .1 .5];
% cfg.planparams.U = repmat({{Ugrid, Ugrid}}, 1, mcfg.nagents);
% cfg.planparams.n = 200;
% % cfg.planparams.gametype = 'max';
% cfg.planparams.rob = 0;
% % cfg.planparams.maxpredlength = 1;
% cfg.planparams.verb = 2;
% cfg.visualize = 2;
% cfg.viscfg.pause = .01;
% cfg.datadir = pwd;
% cfg.maxsteps = 40;
% cfg.verb = 3;
% cfg.saveplandata = 0;
% 
% %% run in case 1
% cfg.reset = x0{1}; cfg.run = 1; cfg.datafile = 'magenmpc_malin_2d_vrob'; magenmpc(cfg);
% %% run in case 2
% cfg.reset = x0{2}; cfg.run = 1; cfg.datafile = 'magenmpc_malin_2d_105vrob'; magenmpc(cfg);
% %% run in case 3
% cfg.reset = x0{3}; cfg.run = 1; cfg.datafile = 'magenmpc_malin_2d_15vrob'; magenmpc(cfg);


%% show world -- no longer needed, implemented in _plot function
% cfg.reset = x0{3}; cfg.run = 1; cfg.visualize = 2; cfg.maxsteps = 0;
% cfg.datafile = 'dummy'; 
% magenmpc(cfg);
% newax = copyobj(subplot(2, 2, [1 2]), figure);
% set(newax, 'units', 'normalized', 'position', [0 0 1 1]);
% axis square off
% set(gcf, 'renderer', 'zbuffer');
% saveplot(gcf, 'd:\Work\tex\techrep_preliminary\opflocking\img\2d_x0', 'latex');



%% Small config: 4 agents
% 
% mcfg = struct;
% mcfg.type = 'doubleint';
% mcfg.nagents = 4;
% mcfg.dim = 2;
% mcfg.range = 5;
% mcfg.flock_dist = 5;
% mcfg.flock_maxdvel = 5;
% % mcfg.flock_alpha = .5;
% % mcfg.flock_beta = .5;
% mcfg.Ts = 0.5;
% 
% m = malinear_problem('model', mcfg);
% 
% gridstep = mcfg.range * .39;
%  % setup grid structure in 2D
% gr = zeros(2, 2, 2); 
% gr(1, :, :) = [-1 -1; 1 1];
% gr(2, :, :) = [1 -1; 1 -1];
% 
% % initial position (always the same)
% p0 = gr .* gridstep;
% % velocities don't matter for robust graph, set them to 0
% x = [p0; 0.*p0];
% [Grob, rhorob, lambda2rob] = malinear_problem('minrobgraph', m, x(:));
% 
% % maximal norm of velocity disagreement vector given by theory
% vrob = lambda2rob * rhorob / sqrt(2) / sqrt(sum(abs(gr(:))));
% % some velocities violating condition to various extents
% v0xy = [0.5 1] * vrob;  
% 
% % compose initial states
% n0 = length(v0xy);
% x0 = cell(n0, 1);
% for i = 1:n0,
%     x = [p0; gr .* v0xy(i)];
%     x0{i} = x(:); 
% end;
% 
