function bike_fzqi_resstudy(mode, cfg, exp)
% Discrete double integrator. Resolution study with fuzzy Q-iteration
% These experiments are:
%   - run in Feb 2009, 
%   - on the new DCSC desktop machines (Dell Optiplex 755)

% What experiments to process and in what mode (defaults)
if nargin < 1, mode = 'run'; end;
if nargin < 2, cfg = struct; end;

% default (main) CFG
CFG = struct;
CFG.(mode) = 1;
CFG.problem = 'bike_problem'; 
CFG.gamma = 0.98;
CFG.model_params = {'det=1'};
CFG.N = 3:18;
CFG.M = 3;
CFG.maxiter = 5000;
CFG.eps = 1e-5;
% perf eval config
CFG.X0 = 'p2s15';
CFG.mc_eps = 0.001;
% datafile and output config
CFG.dfprefix = 'fzbike_resstudy_';
CFG.datadir = [pwd '\'];
CFG.datafile = 'bike_fzqi_resstudy_scorecache';
CFG.silent = 0;     % experiments can be quite long, it's useful to see what's going on
% fig config
CFG = setfigprop(CFG, 'addfields');
CFG.xgridfigure = 'off';      % since qiresstudy makes it 'on' by default
CFG.legendprefix = ''; CFG.legendpostfix = '';
CFG.xlabel = [];

% install any non-default config; preserve the default
cfg = parseconfig(cfg, CFG);

% old saturation experiment
ecfg = cfg;
ecfg.datafile = 'bike_fzqi_resstudy_scorecache';
if checkrunexp('old', exp), qiresstudy(ecfg); end;

% May 2010, terminal state handling changed
ecfg = cfg;
ecfg.term = 'zero';
ecfg.dfinfix = 'termzero';
ecfg.datafile = 'bike_fzqi_resstudy_termzero_scorecache';
if checkrunexp('termzero', exp), qiresstudy(ecfg); end;

end