function lspidc_h0g97ortho(exp, mode, c)
% Initial policy experiments for DC motor 
% using gamma=0.97 and the Q&R "standard" 
% These are updated experiments that supersede lspidc_h0

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'all'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% main cfg
cfg = struct;
cfg.(mode) = 1;
cfg.gamma = .97;
cfg.problem = 'dc_problem';
cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'}; % Larger LQR weights, lin model
cfg.datadir = pwd;
cfg.show = {'gamma', 'rep'};
% sample config is constant
cfg.adaptiveNs = 1;
cfg.Ns = 10000;  % sample generation method will differ according to experiment   
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.maxiter  = 50;
cfg.startat = 1; cfg.repeat = 5;
% perf eval
cfg.perf = 1;                       % turn off for now, we care about iter only
cfg.X0 = 'coarse';                  % when evaluating performance, use coarse grid
cfg.mc_eps = 0.1;      
% replay
cfg.x0 = [pi;0]; cfg.tend = 1.5;
% plot
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.plot = {'J', 'iter'};
%     if any(strcmp(exp, '*opt')),    cfg.savefig = 'dch0opt';
%     elseif any(strcmp(exp, '*pd')), cfg.savefig = 'dch0pd';
%     else                            cfg.savefig = 'dch0';
%     end;
%     cfg.savedir = 'D:\Work\tex\reports\2008-02-11_3wr\img\';
% %     cfg.plottarget = 'latex'; 
    cfg.ylim_matrix = [-350 -250; 0 50];
    cfg.figsize = [450 350];
end;
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% Config for using NO initial policy (derived from theta0=0, identically 0)
nilcfg = cfg;
nilcfg.infix = 'h0nil';
% Config for using OPTIMAL policy (discounted LQR controller)
hcfg_opt = cfg;
model = dc_problem('model', cfg.model_params{:}); 
K = gammadare(struct('model', model, 'gamma', cfg.gamma));
hcfg_opt.lspi_params.h0 = @lsf;
hcfg_opt.lspi_params.h0args = {K, model.maxu};
hcfg_opt.infix = 'h0opt';
% Config for using PD policy (discounted LQR controller)
hcfg_pd = cfg;
hcfg_pd.lspi_params.h0 = @pdstatic;
hcfg_pd.lspi_params.h0args = {[21.1765 0.6514], model.maxu, 0};
hcfg_pd.infix = 'h0pd';

% % code to plot the optimal policy
% pcfg.genh = 1; pcfg.policy = @lsf; pcfg.policyargs = {[], K, model.maxu};
% dc_plot(pcfg);
% return;


expout = [];        % reset figure additions
% --------------------------------------
% -------- TRIANG experiments
% --------------------------------------
% TRIANGULAR discrete / optimal policy
thexp_opt = hcfg_opt;
thexp_opt.type     = 'triang'; thexp_opt.miscinfo = 'th_opt';
thexp_opt.N        = 9; thexp_opt.M = [3 5];
thexp_opt.method   = 'randdisc';
thexp_opt.addtocrt = 0; thexp_opt.expprefix = 'h_0=h*,';     % only effective in stats+plot mode
if checkrunexp(thexp_opt.miscinfo, exp), expout = lspiexp(thexp_opt); end;
% TRIANGULAR discrete / PD policy
thexp_pd = hcfg_pd;
thexp_pd.type     = 'triang'; thexp_pd.miscinfo = 'th_pd';
thexp_pd.N        = 9; thexp_pd.M = [3 5];
thexp_pd.method   = 'randdisc';
thexp_pd.usesamples = thexp_opt;
thexp_pd.addtocrt = expout; thexp_pd.expprefix = 'h_0=PD,';     % only effective in stats+plot mode
if checkrunexp(thexp_pd.miscinfo, exp), expout = lspiexp(thexp_pd); end;
% TRIANGULAR discrete
texp = nilcfg;
texp.type     = 'triang'; texp.miscinfo = 't_nil';
texp.N        = 9; texp.M = [3 5]; 
texp.method   = 'randdisc';
texp.addtocrt = expout; texp.expprefix = 'no h_0,';     % only effective in stats+plot mode
texp.usesamples = thexp_opt;
if checkrunexp(texp.miscinfo, exp), expout = lspiexp(texp); end;

expout = [];        % reset figure additions
% --------------------------------------
% -------- TRIANG ORTHO experiments
% --------------------------------------
% TRIANGULAR quad / optimal policy
tohexp_opt = hcfg_opt;
tohexp_opt.type     = 'triangortho'; tohexp_opt.miscinfo = 'toh_opt';
tohexp_opt.N        = 9; tohexp_opt.M = [2 3 4];
tohexp_opt.method   = 'rand';
tohexp_opt.addtocrt = 0; tohexp_opt.expprefix = 'h_0=h*,';     % only effective in stats+plot mode
if checkrunexp(tohexp_opt.miscinfo, exp), expout = lspiexp(tohexp_opt); end;
% TRIANGULAR quad / PD policy
tohexp_pd = hcfg_pd;
tohexp_pd.type     = 'triangortho'; tohexp_pd.miscinfo = 'toh_pd';
tohexp_pd.N        = 9; tohexp_pd.M = [2 3 4];
tohexp_pd.method   = 'rand';
tohexp_pd.usesamples = tohexp_opt;
tohexp_pd.addtocrt = expout; tohexp_pd.expprefix = 'h_0=PD,';     % only effective in stats+plot mode
if checkrunexp(tohexp_pd.miscinfo, exp), expout = lspiexp(tohexp_pd); end;
% TRIANGULAR quad
toexp = nilcfg;
toexp.type     = 'triangortho'; toexp.miscinfo = 'to_nil';
toexp.N        = 9; toexp.M = [2 3 4];
toexp.method   = 'rand';
toexp.usesamples = tohexp_opt;
toexp.addtocrt = expout; toexp.expprefix = 'no h_0,';     % only effective in stats+plot mode
if checkrunexp(toexp.miscinfo, exp), expout = lspiexp(toexp); end;

expout = [];        % reset figure additions
% --------------------------------------
% -------- RBF experiments
% --------------------------------------
% RBF discrete H
rhexp_opt = hcfg_opt;
rhexp_opt.type     = 'rbfdisc'; rhexp_opt.miscinfo = 'rh_opt';
rhexp_opt.N        = 9; rhexp_opt.M = [3 5];
rhexp_opt.method   = 'randdisc';
rhexp_opt.usesamples = thexp_opt;
rhexp_opt.addtocrt = 0; rhexp_opt.expprefix = 'h_0=h*,';     % only effective in stats+plot mode
if checkrunexp(rhexp_opt.miscinfo, exp), expout = lspiexp(rhexp_opt); end;
% RBF discrete / PD policy
rhexp_pd = hcfg_pd;
rhexp_pd.type     = 'rbfdisc'; rhexp_pd.miscinfo = 'rh_pd';
rhexp_pd.N        = 9; rhexp_pd.M = [3 5];
rhexp_pd.method   = 'randdisc';
rhexp_pd.usesamples = thexp_opt;
rhexp_pd.addtocrt = expout; rhexp_pd.expprefix = 'h_0=PD,';     % only effective in stats+plot mode
if checkrunexp(rhexp_pd.miscinfo, exp), expout = lspiexp(rhexp_pd); end;
% RBF discrete
rexp = nilcfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r_nil';
rexp.N        = 9; rexp.M = [3 5]; 
rexp.method   = 'randdisc';
rexp.usesamples = thexp_opt;
rexp.addtocrt = expout; rexp.expprefix = 'no h_0,';     % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

expout = [];        % reset figure additions
% --------------------------------------
% -------- RBF ORTHO experiments
% --------------------------------------
% RBF ortho H
rohexp_opt = hcfg_opt;
rohexp_opt.type     = 'rbfortho'; rohexp_opt.miscinfo = 'roh_opt';
rohexp_opt.N        = 9; rohexp_opt.M = [2 3 4];
rohexp_opt.method   = 'rand';
rohexp_opt.usesamples = tohexp_opt;
rohexp_opt.addtocrt = 0; rohexp_opt.expprefix = 'h_0=h*,';     % only effective in stats+plot mode
if checkrunexp(rohexp_opt.miscinfo, exp), expout = lspiexp(rohexp_opt); end;
% RBF ortho / PD policy
rohexp_pd = hcfg_pd;
rohexp_pd.type     = 'rbfortho'; rohexp_pd.miscinfo = 'roh_pd';
rohexp_pd.N        = 9; rohexp_pd.M = [2 3 4];
rohexp_pd.method   = 'rand';
rohexp_pd.usesamples = tohexp_opt;
rohexp_pd.addtocrt = expout; rohexp_pd.expprefix = 'h_0=PD,';     % only effective in stats+plot mode
if checkrunexp(rohexp_pd.miscinfo, exp), expout = lspiexp(rohexp_pd); end;
% RBF ortho
roexp = nilcfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'ro_nil';
roexp.N        = 9; roexp.M = [2 3 4];
roexp.method   = 'rand';
roexp.usesamples = tohexp_opt;
roexp.addtocrt = expout; roexp.expprefix = 'no h_0,';     % only effective in stats+plot mode
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;



