function lspiohdc_h(exp, mode, c)
% DC motor. Online LSPI with explicit policy parameterization

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'all'; end;
if nargin < 2, mode = 'onlineperf'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
% --------------
% main LSPIEXP config
cfg.(mode) = 2;
cfg = setfigprop(cfg, 'addfields');
cfg.startat = 1; cfg.repeat = 5;
% algorithm fixed config
cfg.problem = 'dc_problem';
cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'}; % Larger LQR weights, lin model
cfg.gamma = .97;
% settings for original reward function 
cfg.orig_model_params= {'lqr', 'lin'};
cfg.orig_gamma = .98;
% misc config (output etc)
cfg.datadir = pwd;
cfg.eps = 0;          % zero convergence threshold (force until real convergence)
% approximators main config 
cfg.xspace = 'eq'; cfg.uspace = 'log'; cfg.hxspace = 'eq';
cfg.hmethod = 'rand'; cfg.hNs = 1000;
cfg.datadir = pwd;
% perf eval
cfg.X0 = 'small'; cfg.perf = 1; cfg.mc_eps = 0.1; 
cfg.addtocrt = [];
cfg.quantize = 0;
cfg.plotminmax = 1; cfg.plotconf = 0;  cfg.conftype = 'gauss';
cfg.minmaxtype = 'line';
cfg.forcerecompute = 0;
% replay
cfg.x0 = [pi;0]; cfg.tend = 3;    % for replaying
% plot config
cfg.savedir = 'd:\work\tex\papers\lspionlcon\img\';
cfg.plottarget = 'screen'; cfg.savefig = 'dcoh';
% if strcmp(mode, 'onlineperf'), 
% %     cfg.stats = 1; cfg.plot = {'J'}; 
% 	cfg.figsize = [450 400];
%     if strcmp(exp, 'rd*'),
%         cfg.ylim = [-800 -300];
%     elseif strcmp(exp, 'ro*'),
%         cfg.ylim = [-2000 -300];
%     end;
%         
% end;
if any(strcmp(mode, {'stats', 'plot', 'onlineperf'})), cfg.verb = 0; else cfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% --------------
% LSPIONLINE config
ocfg = cfg;
ocfg.trialtime = 1.5;           % sufficient for stabilization from most (all?) initial states
ocfg.lspi_params.maxtime  = 60*10;          % 10 simulated minutes
ocfg.explordecay = 0.1^(1/ocfg.lspi_params.maxtime);
ocfg.dupd = 100 * 0.005;
ocfg.lspi_params.explor = 1;                % start with this exploration rate
ocfg.lspi_params.dstoretheta = 1;           % store theta once every second
    % decay so that we have 0.1 at the end of the experiment
ocfg.lspi_params.verb = 5;
ocfg.lspi_params.ddisp = 10;
ocfg.lspi_params.dsave = 60;                % save once every minute
ocfg.show = {'rep'};
ocfg.donlineperf = [5 10 15 20:10:50 60:15:120 150 180:60:600]; 
% perf eval
ocfg.forceeval = 1;                         % since no experiment actually converges

% LSPIHONLINE config
ohcfg = ocfg;
ohcfg.alg = 'lspihonline';
ohcfg.infix = 'nilc';
% with constraints
ohconcfg = ocfg;
ohconcfg.alg = 'lspihonline';
ohconcfg.infix = 'mon';
ohconcfg.con = 'type=mon dir=[-1,-1]';
% exact
oexcfg = ocfg;
oexcfg.alg = 'lspionline';
oexcfg.infix = 'exact';

% --------------
% LSPI config
bcfg = cfg;
bcfg.alg = 'lspih';
bcfg.show = {'rep'};
bcfg.maxiter = 200;
bcfg.infix = 'offlh';

expout = 0;   % dummy

% ---- 2009-10-30 BOOK EXPERIMENTS, WITH CHANGED EXPLORATION SCHEDULE
% These are run with default DC motor settings (no changed weights in the reward), on the new
% desktop machines

% exact policies
% 20 experiments 
rdexp = oexcfg;
rdexp.model_params = cfg.orig_model_params;
rdexp.gamma = cfg.orig_gamma;
rdexp.explordecay = 0.1^(1/200);
rdexp.type     = 'rbfdisc'; rdexp.miscinfo = 'rdonlB';
rdexp.N        = 9; rdexp.M = 3;
rdexp.infix = 'exactB';
% rdexp.addtocrt = expout; rdexp.expprefix = 'Implicitly defined h & no prior knowledge';  % only effective in stats+plot mode
if checkrunexp(rdexp.miscinfo, exp), expout = lspiexp(rdexp); end;

% monotonous policies
% 20 experiments 
rdhcexp = ohconcfg;
rdhcexp.model_params = cfg.orig_model_params;
rdhcexp.gamma = cfg.orig_gamma;
rdhcexp.explordecay = 0.1^(1/200);
rdhcexp.type     = 'rbfdisc'; rdhcexp.miscinfo = 'rdhconlB';
rdhcexp.N        = 9; rdhcexp.M = 3;
rdhcexp.hN       = 9;
rdhcexp.infix = 'monB';
% rdhcexp.addtocrt = expout; rdhcexp.expprefix = 'Parameterized h & prior knowledge';  % only effective in stats+plot mode
if checkrunexp(rdhcexp.miscinfo, exp), expout = lspiexp(rdhcexp); end;

% ---- 2009-09-14 BOOK EXPERIMENTS, DEFAULT REWARD FUN
% These are run with default DC motor settings (no changed weights in the reward), on the new
% desktop machines

% exact policies
% 20 experiments available + 20archived
rdexp = oexcfg;
rdexp.model_params = cfg.orig_model_params;
rdexp.gamma = cfg.orig_gamma;
rdexp.type     = 'rbfdisc'; rdexp.miscinfo = 'rdonl2';
rdexp.N        = 9; rdexp.M = 3;
rdexp.infix = 'exact2';
% rdexp.addtocrt = expout; rdexp.expprefix = 'Implicitly defined h & no prior knowledge';  % only effective in stats+plot mode
if checkrunexp(rdexp.miscinfo, exp), expout = lspiexp(rdexp); end;

% explicit policies
% 20 experiments available + 20archived
rdhexp = ohcfg;
rdhexp.model_params = cfg.orig_model_params;
rdhexp.gamma = cfg.orig_gamma;
rdhexp.type     = 'rbfdisc'; rdhexp.miscinfo = 'rdhonl2';
rdhexp.N        = 9; rdhexp.M = 3;
rdhexp.hN       = 9;
rdhexp.infix = 'nilc2';
% rdhexp.addtocrt = expout; rdhexp.expprefix = 'Parameterized h & no prior knowledge';  % only effective in stats+plot mode
if checkrunexp(rdhexp.miscinfo, exp), expout = lspiexp(rdhexp); end;

% monotonous policies
% 20 experiments available + 20archived (warning: #9 duplicate with #21 -- after eliminating outlier)
rdhcexp = ohconcfg;
rdhcexp.model_params = cfg.orig_model_params;
rdhcexp.gamma = cfg.orig_gamma;
rdhcexp.type     = 'rbfdisc'; rdhcexp.miscinfo = 'rdhconl2';
rdhcexp.N        = 9; rdhcexp.M = 3;
rdhcexp.hN       = 9;
rdhcexp.infix = 'mon2';
% rdhcexp.addtocrt = expout; rdhcexp.expprefix = 'Parameterized h & prior knowledge';  % only effective in stats+plot mode
if checkrunexp(rdhcexp.miscinfo, exp), expout = lspiexp(rdhcexp); end;

% ---- DISCRETE-ACTION APPROXIMATORS

% ONLINE RBF DISCRETE, exact policies
% datafiles 1-5 (exact PImpr, online), were not rerun, but copied from lspiodc_dupd and renamed
% 20 experiments available
rdexp = oexcfg;
rdexp.type     = 'rbfdisc'; rdexp.miscinfo = 'rdonl';
rdexp.N        = 9; rdexp.M = 3;
rdexp.addtocrt = expout; rdexp.expprefix = 'Implicitly defined h & no prior knowledge';  % only effective in stats+plot mode
if checkrunexp(rdexp.miscinfo, exp), expout = lspiexp(rdexp); end;

% ONLINE RBF DISCRETE, explicit policies
% 20 experiments available
rdhexp = ohcfg;
rdhexp.type     = 'rbfdisc'; rdhexp.miscinfo = 'rdhonl';
rdhexp.N        = 9; rdhexp.M = 3;
rdhexp.hN       = 9;
rdhexp.addtocrt = expout; rdhexp.expprefix = 'Parameterized h & no prior knowledge';  % only effective in stats+plot mode
if checkrunexp(rdhexp.miscinfo, exp), expout = lspiexp(rdhexp); end;

% ONLINE RBF DISCRETE, monotonous policies
% 20 experiments available
rdhcexp = ohconcfg;
rdhcexp.type     = 'rbfdisc'; rdhcexp.miscinfo = 'rdhconl';
rdhcexp.N        = 9; rdhcexp.M = 3;
rdhcexp.hN       = 9;
rdhcexp.addtocrt = expout; rdhcexp.expprefix = 'Parameterized h & prior knowledge';  % only effective in stats+plot mode
if checkrunexp(rdhcexp.miscinfo, exp), expout = lspiexp(rdhcexp); end;


% ---- CONTINUOUS-ACTION APPROXIMATORS

% ONLINE RBF ORTHOPOLY, exact policies
% These datafiles (exact PImpr, online), were not rerun, but copied from lspiodc_dupd and renamed
% 5 experiments available (not used in chapter)
roexp = oexcfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'roonl';
roexp.N        = 9; roexp.M = 2;
roexp.hN       = 9;
roexp.addtocrt = expout; roexp.expprefix = 'Ortho, no prior knowledge';  % only effective in stats+plot mode
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;

% ONLINE RBF ORTHOPOLY, explicit policies
% 5 experiments available (not used in chapter)
rohexp = ohcfg;
rohexp.type     = 'rbfortho'; rohexp.miscinfo = 'rohonl';
rohexp.N        = 9; rohexp.M = 2;
rohexp.hN       = 9;
rohexp.addtocrt = expout; rohexp.expprefix = 'Ortho, parameterized h';  % only effective in stats+plot mode
if checkrunexp(rohexp.miscinfo, exp), expout = lspiexp(rohexp); end;

% ONLINE RBF ORTHOPOLY, monotonous policies
% 20 experiments available (by mistake...) (not used in chapter)
rohcexp = ohconcfg;
rohcexp.type     = 'rbfortho'; rohcexp.miscinfo = 'rohconl';
rohcexp.N        = 9; rohcexp.M = 2;
rohcexp.hN       = 9;
rohcexp.addtocrt = expout; rohcexp.expprefix = 'Ortho, monotonous h';  % only effective in stats+plot mode
if checkrunexp(rohcexp.miscinfo, exp), expout = lspiexp(rohcexp); end;

% % BASELINE, offline RBF ORTHOPOLY experiment (not used in chapter)
% robexp = bcfg;
% robexp.type     = 'rbfortho'; robexp.miscinfo = 'robaseline';
% robexp.eps      = 1e-5;     % this one will never actually converge to 0
% robexp.N        = 9; robexp.M = 2;
% robexp.method   = 'rand';
% robexp.Ns       = 20000;  % many samples to get close to the best achievable solution
% robexp.addtocrt = expout; robexp.expprefix = 'Offline'; % only effective in stats+plot mode
% robexp.infix    = 'dupd';
% if checkrunexp(robexp.miscinfo, exp), expout = lspiexp(robexp); end;


end



