
% adhoc script to test functionality of the edro_rtproblem function
% implements proportional control

% ref, controller gain, tolerance around ref
Kp = 3; ref = 0;
% Kx = [3 -0.1]; ref = [0 0];
% Kx = [3 -0.1]; ref = [0.5 1];
tol = 0.05;

% create system structure and initialise
sys = edro_rtproblem('system', 'xjoints=5');
sys = edro_rtproblem('init', sys);

njoints = sys.p / 2;
K = 40;
X = nan(sys.p, K+1);
U = nan(sys.q, K+1);
R = nan(1, K+1);

edro_rtproblem('reset', sys, 'type=randpos');
tic;
for k = 1 : K,                       % run for at most 40*h seconds
    x = edro_rtproblem('readx', sys);
%     if ref(2) ~=0 && all(abs(x - ref') < tol),
%         disp('reached');
%         break;
%     end;
    % compute control, picking up only the angles
    u = max(-.5, min(0.5, Kp .* (ref - x(1:2:sys.p))));
%     u = Kx * (ref' - x);
    edro_rtproblem('writeu', sys, u);
    X(:, k) = x;
    U(:, k) = u;
    if k > 1, 
        R(k) = sys.fun(sys, X(:, k-1), U(:, k-1), X(:, k));
    end;
    while toc < sys.Ts, end;          % wait for the next sample
    tic;
end;
% read final state and reward
X(:, k+1) = edro_rtproblem('readx', sys);
R(:, k+1) = sys.fun(sys, X(:, k), U(:, k), X(:, k+1));

h.t = sys.Ts .* (0:K); 
h.x = X; 
h.u = U; 
h.r = R;
plothistory(h);

sys = edro_rtproblem('close', sys);