% Parameter study for llrfittedqi on the invpend problem

%% #### General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'llrfittedqi';
BCFG.verb = 5;

% algorithm
ACFG = struct;
ACFG.problem = 'hillcar_problem';
ACFG.gamma = .95;
ACFG.maxiter  = 500;
ACFG.eps = 0.01;        % make this small since the rewards have small magnitude in the hillcar problem
ACFG.epsper = 0.01;
ACFG.datadir = 'd:\work\m\approxrl\systems\hillcar\llrfqi_hc_paramstudy\';
ACFG.U = {[-4 4]};
ACFG.verb = 2;
% perf eval options
ACFG.mc_eps = .01;
ACFG.mc_N = 1;
ACFG.X0 = 'posspeed';

% move to appropriate directory
cd(ACFG.datadir);

%% #### Experiment 1: effect of knn and #samples // grid sampling
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'kNsgrid';
bcfg.idfields = {'alg', 'problem', 'knn', 'samplesmethod', 'samplesxgrids'};
bcfg.repeat = 1;
% perf eval options
bcfg.in = 'knn';

acfg = ACFG;
acfg.knn = [3 5 7 10 15 20];
acfg.samples.method = 'grid';
acfg.samples.gridfun = @symequidgrid;
acfg.samples.xgrids = {[15, 15], [31 31], [51 51], [71 71]};
acfg.samples.ugrids = acfg.U;

%% Run
processbatch(bcfg, acfg);

%% Eval, separately for each grids
bcfg.out = 'score';

bcfg.addtocrt = 0; bcfg.plotindex = 1;
bcfg.legendprefix = '15x15x2 grid';
acfg.samples.xgrids = [15 15];
evalbatch(bcfg, acfg);

bcfg.addtocrt = 1; bcfg.plotindex = 2;
bcfg.legendprefix = '31x31x2 grid';
acfg.samples.xgrids = [31 31];
evalbatch(bcfg, acfg);

bcfg.addtocrt = 1; bcfg.plotindex = 3;
bcfg.legendprefix = '51x51x2 grid';
acfg.samples.xgrids = [51 51];
evalbatch(bcfg, acfg);

bcfg.addtocrt = 1; bcfg.plotindex = 4;
bcfg.legendprefix = '71x71x2 grid';
acfg.samples.xgrids = [71 71];
evalbatch(bcfg, acfg);


%% Eval across grids, for a given value of k
bcfg.out = 'score';
bcfg.in = 'samplesxgrids';
bcfg.addtocrt = 0; bcfg.plotindex = 1;

acfg.knn = 7;
acfg.samples.xgrids = {[15, 15], [31 31], [51 51], [71 71]};

evalbatch(bcfg, acfg);


%% #### Experiment 2: effect of knn and #samples // unif random sampling

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'kNsrand';
bcfg.idfields = {'alg', 'problem', 'knn', 'samplesmethod', 'samplesN'};
bcfg.repeat = 10;
bcfg.reusesamples = 1;
bcfg.in = 'knn';

acfg = ACFG;
acfg.knn = [3 5 7 10 15 20];
acfg.samples.method = 'randdisc';
acfg.samples.ugrids = acfg.U;


%% Run
% sample counts the same as in grids above, except now they are random
% we run separately for each sample number, to be able to appropriately reuse samples

acfg.samples.N = 15^2 * 2; processbatch(bcfg, acfg);
acfg.samples.N = 31^2 * 2; processbatch(bcfg, acfg);
acfg.samples.N = 51^2 * 2; processbatch(bcfg, acfg);
acfg.samples.N = 71^2 * 2; processbatch(bcfg, acfg);

%% Eval separately for each sample set
bcfg.out = 'score';

acfg.samples.N = 15^2 * 2;
bcfg.addtocrt = 0; bcfg.plotindex = 1;
bcfg.legendprefix = '15^2*2 rand';
evalbatch(bcfg, acfg);

acfg.samples.N = 31^2 * 2;
bcfg.addtocrt = 1; bcfg.plotindex = 2;
bcfg.legendprefix = '31^2*2 rand';
evalbatch(bcfg, acfg);

acfg.samples.N = 51^2 * 2;
bcfg.addtocrt = 2; bcfg.plotindex = 3;
bcfg.legendprefix = '51^2*2 rand';
evalbatch(bcfg, acfg);

acfg.samples.N = 71^2 * 2;
bcfg.addtocrt = 3; bcfg.plotindex = 4;
bcfg.legendprefix = '71^2*2 rand';
evalbatch(bcfg, acfg);

%% Eval across grids, for a given value of k
bcfg.in = 'samplesN';
bcfg.out = 'score';
bcfg.addtocrt = 0; bcfg.plotindex = 1;

acfg.knn = 7;
acfg.samples.N = [15 31 51 71].^2 * 2;

evalbatch(bcfg, acfg);


