% Parameter study for llrfittedqi on the invpend problem

%% #### General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'llrfittedqi';
BCFG.verb = 5;

% algorithm
ACFG = struct;
ACFG.problem = 'ipsetup_problem';
ACFG.gamma = .98;
ACFG.maxiter  = 500;
ACFG.datadir = 'd:\work\m\approxrl\systems\invpend_setup\llrfqi_ip_paramstudy\';
ACFG.U = {[-3 0 3]};
ACFG.verb = 2;
% perf eval options
ACFG.mc_eps = .1;
ACFG.mc_N = 1;
ACFG.X0 = 'smallsym';

% move to appropriate directory
cd(ACFG.datadir);

%% #### Experiment 1: effect of knn and #samples // grid sampling
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'kNsgrid';
bcfg.idfields = {'alg', 'problem', 'knn', 'samplesmethod', 'samplesxgrids'};
bcfg.repeat = 1;
% perf eval options
bcfg.in = 'knn';

acfg = ACFG;
acfg.knn = [2 3 4 5 6 7 8 9 10 15 20];
acfg.samples.method = 'grid';
acfg.samples.gridfun = @symequidgrid;
acfg.samples.xgrids = {[15, 15], [31 31], [51 51], [71 71]};
acfg.samples.ugrids = acfg.U;

%% Run
processbatch(bcfg, acfg);

%% Eval, separately for each grids
bcfg.out = 'score';

acfg.samples.xgrids = [15 15];
bcfg.addtocrt = 0; bcfg.plotindex = 1;
bcfg.legendprefix = '15x15x3 grid';
evalbatch(bcfg, acfg);

acfg.samples.xgrids = [31 31];
bcfg.addtocrt = 1; bcfg.plotindex = 2;
bcfg.legendprefix = '31x31x3 grid';
evalbatch(bcfg, acfg);

acfg.samples.xgrids = [51 51];
bcfg.addtocrt = 1; bcfg.plotindex = 3;
bcfg.legendprefix = '51x51x3 grid';
evalbatch(bcfg, acfg);

acfg.samples.xgrids = [71 71];
bcfg.addtocrt = 1; bcfg.plotindex = 4;
bcfg.legendprefix = '71x71x3 grid';
evalbatch(bcfg, acfg);


%% Eval across grids, for a given value of k
bcfg.out = 'iter';
bcfg.in = 'samplesxgrids';
bcfg.addtocrt = 0; bcfg.plotindex = 1;

acfg.knn = 10;
acfg.samples.xgrids = {[15, 15], [31 31], [51 51], [71 71]};

evalbatch(bcfg, acfg);


%% #### Experiment 2: effect of knn and #samples // unif random sampling

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'kNsrand';
bcfg.idfields = {'alg', 'problem', 'knn', 'samplesmethod', 'samplesN'};
bcfg.repeat = 10;
bcfg.reusesamples = 1;
bcfg.in = 'knn';

acfg = ACFG;
acfg.knn = [3 5 7 10 15 20];
acfg.samples.method = 'randdisc';
acfg.samples.ugrids = acfg.U;


%% Run
% sample counts the same as in grids above, except now they are random
% we run separately for each sample number, to be able to appropriately reuse samples

acfg.samples.N = 15^2 * 3;        
processbatch(bcfg, acfg);

acfg.samples.N = 31^2 * 3;
processbatch(bcfg, acfg);

acfg.samples.N = 51^2 * 3;
processbatch(bcfg, acfg);

acfg.samples.N = 71^2 * 3;
processbatch(bcfg, acfg);

%% Eval separately for each sample set
bcfg.out = 'score';

acfg.samples.N = 15^2 * 3;
bcfg.addtocrt = 0; bcfg.plotindex = 1;
bcfg.legendprefix = '15^2*3 rand';
evalbatch(bcfg, acfg);

acfg.samples.N = 31^2 * 3;
bcfg.addtocrt = 1; bcfg.plotindex = 2;
bcfg.legendprefix = '31^2*3 rand';
evalbatch(bcfg, acfg);

acfg.samples.N = 51^2 * 3;
bcfg.addtocrt = 2; bcfg.plotindex = 3;
bcfg.legendprefix = '51^2*3 rand';
evalbatch(bcfg, acfg);

acfg.samples.N = 71^2 * 3;
bcfg.addtocrt = 3; bcfg.plotindex = 4;
bcfg.legendprefix = '71^2*3 rand';
evalbatch(bcfg, acfg);

%% Eval across grids, for a given value of k
bcfg.in = 'samplesN';
bcfg.out = 'iter';
bcfg.addtocrt = 0; bcfg.plotindex = 1;

acfg.knn = 10;
acfg.samples.N = [15 31 51 71].^2 * 3;

evalbatch(bcfg, acfg);

%% #### Experiment 3: effect of weighting function // fixed grid sampling, k varies but not so much
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'wexp';
bcfg.idfields = {'alg', 'problem', 'knn', 'wfun', 'samplesmethod', 'samplesxgrids'};
bcfg.repeat = 1;
% perf eval options
bcfg.in = 'knn';

acfg = ACFG;
acfg.knn = [3 5 7 10 15 20];
acfg.wfun = {'unif', 'triang', 'quad', 'cube'};
acfg.samples.method = 'grid';
acfg.samples.gridfun = @symequidgrid;
acfg.samples.xgrids = [51 51];          % note this is used as a single vector
acfg.samples.ugrids = acfg.U;

%% Run
bcfg.mode = 'cleanup'; acfg.cleanuplevel = 3;

processbatch(bcfg, acfg);

%% Eval
bcfg.out = 'score';

acfg.wfun = 'unif';
bcfg.addtocrt = 0; bcfg.plotindex = 1;
bcfg.legendprefix = 'wfun=unif';
evalbatch(bcfg, acfg);

acfg.wfun = 'triang';
bcfg.addtocrt = 1; bcfg.plotindex = 2;
bcfg.legendprefix = 'wfun=triangular';
evalbatch(bcfg, acfg);

acfg.wfun = 'quad';
bcfg.addtocrt = 1; bcfg.plotindex = 3;
bcfg.legendprefix = 'wfun=quadratic';
evalbatch(bcfg, acfg);

acfg.wfun = 'cube';
bcfg.addtocrt = 1; bcfg.plotindex = 4;
bcfg.legendprefix = 'wfun=tricubic';
evalbatch(bcfg, acfg);


