function lspios_trialtime(exp, mode, c)
% Real inverted pendulum. Using trajectories for samples

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'ronl'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
% --------------
% main LSPIEXP config
cfg.(mode) = 2;
cfg.startat = 1; cfg.repeat = 5;
% algorithm fixed config
cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.98;
cfg.trialtime = [];
% misc config (output etc)
cfg.datadir = pwd;
cfg.eps = 0;                            % zero convergence threshold (force until real convergence)
cfg.useprefixes = 0;
% approximator main config
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
cfg.datadir = pwd;
% perf eval
cfg.forcerecompute = 0;
cfg.X0 = 'small'; cfg.perf = 1; cfg.mc_eps = 0.1; 
cfg.minmaxtype = 'area';
% replay
cfg.x0 = [pi;0]; cfg.tend = 3;    % for replaying
% plot config
cfg.fixedfigname = 0;
cfg.plottarget = 'screen'; cfg.savefig = 'tt';
cfg.plotminmax = 0; cfg.plotconf = 0;
if strcmp(mode, 'plot'), 
    cfg.stats = 1; cfg.plot = {'J'}; 
%     cfg.figsize = [450 350];
end;
% if strcmp(mode, 'onlineperf'), 
%     cfg.ylim = [-2400 -1400];
% end;
if any(strcmp(mode, {'stats', 'plot', 'onlineperf'})), cfg.verb = 0; else cfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% --------------
% LSPIONLINE config
ocfg = cfg;
ocfg.alg = 'lspionline';
ocfg.lspi_params.maxtime  = 60*10;          % 10 simulated minutes
ocfg.explordecay = 0.1^(1/ocfg.lspi_params.maxtime);
ocfg.dupd = 10 * 0.005;
ocfg.lspi_params.explor = 1;                % start with this exploration rate
ocfg.lspi_params.dstoretheta = 1;           % store theta once every second
ocfg.lspi_params.verb = 5;
ocfg.lspi_params.ddisp = 10;
ocfg.lspi_params.dsave = 60;                % save once every minute
ocfg.show = {'trialtime', 'rep'};
ocfg.donlineperf = [15:15:90 120:60:600]; 
% perf eval
ocfg.forceeval = 1;                         % since no experiment actually converges

expout = [];   % dummy

% ONLINE, RBF discrete experiment
% 20 experiments available
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl';
rexp.N        = 11; rexp.M = 3;
if isempty(cfg.trialtime),      % otherwise use cfg.trialtime (it's already in rexp)
    rexp.trialtime = [0.3 0.75 1.5 3 6 12]; 
end;
rexp.addtocrt = []; 
if cfg.useprefixes, rexp.expprefix = 'Online discrete'; end;
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

% ONLINE, RBF discrete experiment; the only difference from the above is that this is run on
% the new DCSC desktop
% 0 experiments available
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl2';
rexp.N        = 11; rexp.M = 3;
if isempty(cfg.trialtime),      % otherwise use cfg.trialtime (it's already in rexp)
    rexp.trialtime = [0.3 0.75 1.5 3 6 12]; 
end;
rexp.infix    = 'tt2';
if cfg.useprefixes, rexp.expprefix = 'Online discrete'; end;
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;


% ONLINE, RBF discrete experiment #3;
% run on the new DCSC desktop, and with an explor schedule that reaches .1
% at T=200
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronlB';
rexp.N        = 11; rexp.M = 3;
rexp.explordecay = 0.1^(1/200);  
if isempty(cfg.trialtime),      % otherwise use cfg.trialtime (it's already in rexp)
    rexp.trialtime = [0.3 0.75 1.5 3 6 12]; 
end;
rexp.infix    = 'ttB';
if cfg.useprefixes, rexp.expprefix = 'Online discrete'; end;
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;



% ===================================
% Old experiments, no longer used

% ONLINE, RBF ORTHO experiment
% 5 experiments available (not used in chapter, old)
roexp = ocfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'roonl';
roexp.N        = 11; roexp.M = 2;
if isempty(cfg.trialtime),      % otherwise use cfg.trialtime (it's already in rexp)
    roexp.trialtime = [0.3 0.75 1.5 3 6 12]; 
end;
roexp.addtocrt = []; 
if cfg.useprefixes, roexp.expprefix = 'Online ortho'; end; 
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;


end



