%% Optimistic planning experiments for the inverted pendulum
function op_enhancements_ip(mode, expids, custombcfg, customacfg, customplanparams)

%% General config
% clear all; close all;

% Stochastic: batch experiment
BCFGSTOCH = struct;
BCFGSTOCH.alg = 'genmpc';
BCFGSTOCH.verb = 5;
BCFGSTOCH.in = 'planparamsn';
BCFGSTOCH.idfields = {'alg', 'problem', 'planalg', 'X', 'planparamsn'};
BCFGSTOCH.startat = 1; BCFGSTOCH.repeat = 20;
BCFGSTOCH.addtocrt = []; BCFGSTOCH.plotindex = 0;
BCFGSTOCH.skipcache = 1;         % we don't need to cache the results, they are easy to compute
%
% Stochastic: algorithm
ACFGSTOCH = struct;
ACFGSTOCH.problem = 'ipsetup_problem';
ACFGSTOCH.model_params = {'nlqr', 'type=ip maxu=3 stoch_prob=[.6,.4] stoch_gain=[1,.7]', 'Ts=0.05'};
ACFGSTOCH.online = 1;
ACFGSTOCH.gamma = 0.95;
ACFGSTOCH.planalg = 'opss';
ACFGSTOCH.planparams.L = 2;
ACFGSTOCH.planparams.U = [-3 0 3];
ACFGSTOCH.trials = 1;
ACFGSTOCH.maxsteps = 80;
ACFGSTOCH.reset.type = 'seq';
ACFGSTOCH.verb = 3;
ACFGSTOCH.datadir = 'd:\work\m\approxrl\systems\invpend_setup\op_enhancements_ip';
ACFGSTOCH.X = 'stableeq';

% Determ: batch
BCFGDET = BCFGSTOCH;
BCFGDET.repeat = 1;
%
% Determ: algorithm
ACFGDET = struct;
ACFGDET.problem = 'ipsetup_problem';
ACFGDET.model_params = {'nlqr', 'type=ip maxu=3', 'Ts=0.05'};
ACFGDET.online = 1;
ACFGDET.gamma = 0.95;
ACFGDET.planalg = 'opd';
ACFGDET.planparams.U = [-3 0 3];
ACFGDET.trials = 1;
ACFGDET.maxsteps = 80;         % TODO change this?
ACFGDET.reset.type = 'seq';
ACFGDET.verb = 3;
ACFGDET.datadir = 'd:\work\m\approxrl\systems\invpend_setup\op_enhancements_ip';
ACFGDET.X = 'stableeq';



% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;
if nargin < 5, customplanparams = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)) || any(strcmp(expids, {'all', 'any'})),  % if experiment is requested
            bcfg = parseconfig(custombcfg, bcfg); 
            acfg = parseconfig(customacfg, acfg);
            acfg.planparams = parseconfig(customplanparams, acfg.planparams);
            cd(acfg.datadir); 
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            elseif any(strcmp(mode, {'regret', 'exectime', 'plantime', 'depth', 'return'})), 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end

%% STOCH: Baseline: no data reuse

bcfg = BCFGSTOCH;
bcfg.mode = 'run';
bcfg.idinfix = 'noreuse';
bcfg.legendprefix = 'No data reuse';

acfg = ACFGSTOCH;
acfg.planparams.n = [25 50 100 250 500 750 1000];

execbatchexp(bcfg.idinfix);


%% STOCH: Tree reuse

bcfg = BCFGSTOCH;
bcfg.mode = 'run';
bcfg.idinfix = 'reusetree';
bcfg.legendprefix = 'Tree reuse';

acfg = ACFGSTOCH;
acfg.planparams.n = [25 50 100 250 500 750 1000];
acfg.planparams.reusetree = 1;
acfg.planparams.reusetree_maxsize = 0;  % no limit

execbatchexp(bcfg.idinfix);


%% STOCH: Memory: study of effect when unlimited size

bcfg = BCFGSTOCH;
bcfg.mode = 'run';
bcfg.idinfix = 'usemem';
bcfg.legendprefix = 'Memory use';
bcfg.idfields = {BCFGSTOCH.idfields{:}, 'planparamslipschitz', 'planparamsmem_maxsize'};
bcfg.in = 'planparamslipschitz';

acfg = ACFGSTOCH;
acfg.planparams.n = [25 50 100 250 500 750 1000];
acfg.planparams.usemem = 1;
acfg.planparams.mem_maxsize = 0;            % no limit
acfg.planparams.lipschitz = [0.01 0.1 0.5 1 5];

% note we use the 'usemem' infix so that experiments are reused between this and the set below, but
% the actual ID is different
execbatchexp('lip');    


%% STOCH: Memory: influence of memory size

bcfg = BCFGSTOCH;
bcfg.mode = 'run';
bcfg.idinfix = 'usemem';
bcfg.legendprefix = 'Memory use';
bcfg.idfields = {BCFGSTOCH.idfields{:}, 'planparamslipschitz', 'planparamsmem_maxsize'};
bcfg.in = 'planparamsmem_maxsize';

acfg = ACFGSTOCH;
acfg.planparams.n = [25 50 100 250 500 750 1000];
acfg.planparams.usemem = 1;
acfg.planparams.lipschitz = 0.5;        % a good value from the experiment above
acfg.planparams.mem_maxsize = [0 0.25 0.5 1 2];     % 0 should be available from above

execbatchexp('memsize');


%% DET: Baseline: no data reuse

bcfg = BCFGDET;
bcfg.mode = 'run';
bcfg.idinfix = 'detnoreuse';
bcfg.legendprefix = 'No data reuse';

acfg = ACFGDET;
acfg.planparams.n = [10:10:100 200:100:500];

execbatchexp(bcfg.idinfix);


%% DET: Tree reuse

bcfg = BCFGDET;
bcfg.mode = 'run';
bcfg.idinfix = 'detreusetree';
bcfg.legendprefix = 'Tree reuse';

acfg = ACFGDET;
acfg.planparams.n = [10:10:100 200:100:500];
acfg.planparams.reusetree = 1;
acfg.planparams.reusetree_maxsize = 0;  % no limit

execbatchexp(bcfg.idinfix);

%% DET: Memory: study of effect when unlimited size

bcfg = BCFGDET;
bcfg.mode = 'run';
bcfg.idinfix = 'detusemem';
bcfg.legendprefix = 'Memory use';
bcfg.idfields = {BCFGDET.idfields{:}, 'planparamslipschitz', 'planparamsmem_maxsize'};
bcfg.in = 'planparamslipschitz';

acfg = ACFGDET;
acfg.planparams.n = [10:10:100 200:100:500];
acfg.planparams.usemem = 1;
acfg.planparams.mem_maxsize = 0;            % no limit
acfg.planparams.lipschitz = [0.01 0.1 0.5 1 5];

% note we use the 'usemem' infix so that experiments are reused between this and the set below, but
% the actual ID is different
execbatchexp('detlip'); 

%% DET: Memory: influence of memory size

bcfg = BCFGDET;
bcfg.mode = 'run';
bcfg.idinfix = 'detusemem';
bcfg.legendprefix = 'Memory use';
bcfg.idfields = {BCFGSTOCH.idfields{:}, 'planparamslipschitz', 'planparamsmem_maxsize'};
bcfg.in = 'planparamsmem_maxsize';

acfg = ACFGDET;
acfg.planparams.n = [10:10:100 200:100:500];
acfg.planparams.usemem = 1;
acfg.planparams.lipschitz = 0.5;        % a good value from the experiment above
acfg.planparams.mem_maxsize = [0 0.25 0.5 1 2];     % 0 should be available from above

execbatchexp('detmemsize');


end
