function [xplus, rplus, terminal] = resonatingarm_mdp(m, x, u)
% Does not support: noise, refmodel on input; pdf on output

% saturate input
u = max(-m.maxu, min(m.maxu, u));

% run numerical integration over sampling period
odey = m.odesolver(@resonatingarm_trans, m.odet, x, m, u);
xplus = odey(end, :)';

% bound state for the sake of algos that rely on this
xplus = max(-m.maxx, min(m.maxx, xplus));

% compute reward
if m.rewtype(1) == 'n',
    rplus = 1 - ((x-m.xgoal)' * m.Qrew * (x-m.xgoal) + m.Rrew * u^2) / m.nonnorm_maxr;
elseif m.rewtype(1) == 'e',
    rplus = exp(-([xplus-m.xgoal; u]' * m.Crew * [xplus-m.xgoal; u]));
end;

% task is continuing
terminal = 0;

end     % rasetup_singletransition local function


% Transition function of the inverted pendulum setup
function dx = resonatingarm_trans(t, x, m, u)
% pick up variables
theta = x(1);
thetadot = x(2);
% external forces
torquespring = ...
    -( ...
        (m.F0 + m.d*m.k - m.k*m.l0 + ...
            m.k * ((m.dl + m.r1 + m.r2 - m.r2*cos(m.R*theta) - m.r1*cos(theta))^2 ...
                    + (m.r2*sin(m.R*theta) + m.r1*sin(theta))^2)^(1/2)...
        ) ...
        * (m.r1^2*sin(theta) + m.R*m.r2^2*sin(m.R*theta) + m.dl*m.r1*sin(theta) + m.r1*m.r2*sin(theta) ...
             + m.r1*m.r2*sin(theta*(m.R - 1)) + m.R*m.dl*m.r2*sin(m.R*theta) + m.R*m.r1*m.r2*sin(m.R*theta) ...
             - m.R*m.r1*m.r2*sin(theta*(m.R - 1))) ...
    ) / ...
    ((m.dl + m.r1 + m.r2 - m.r2*cos(m.R*theta) - m.r1*cos(theta))^2 + (m.r2*sin(m.R*theta) + m.r1*sin(theta))^2)^(1/2);
% static friction
if thetadot ~= 0, 
    % if moving, full static friction against the direction of movement
    staticfriction = -sign(thetadot) * m.cfs1;
else
    % if not moving, generate at most the static friction equal to external forces
    staticfriction = -sign(torquespring+u) * min(m.cfs1, abs(torquespring+u));
end
% complete friction
friction = -m.cfd1*thetadot + staticfriction;    % first component is dynamic friction
% acceleration: only inclined model supported for now
thetadotdot = ...
    ( torquespring + u + friction + sin(m.angle)*m.g*m.l1*m.lg1*m.m1*sin(-theta+0.5*pi)) ...
     / (m.m1*m.l1^2*m.lg1^2 + m.Ir*m.n^2 + m.I1);
% finally, compose the output
dx = [thetadot; thetadotdot];
end