% config
paintpatches = 1;       % whether to paint patches or just activation of one point
keeptiling = 1;         % whether to keep tiling or rebuild
join = 1;

% tiling config
cfg.init = 0;
cfg.exact = 0;
grids = {0:1, 0:1};
% number of tilings
c = 2;
% number of tiles/params in a tiling
mu = length(grids{1}) * length(grids{2});

if ~keeptiling,     % redo tiling
    [xdata wdata tdim] = tiling_create(grids, [], c, cfg);
end;

% --------------------------
% Compute coordinates of all patches resulting from the intersection of the
% grids
xpatch = sort(xdata{1}(:));
% remove NaNs and duplicate values
xpatch = xpatch(~isnan(xpatch)); xpatch = [xpatch(diff(xpatch) ~= 0); xpatch(end)];

ypatch = sort(xdata{2}(:));
% remove NaNs and duplicate values
ypatch = ypatch(~isnan(ypatch)); ypatch = [ypatch(diff(ypatch) ~= 0); ypatch(end)];


% --------------------------
% Paint patches or one point and its activated tiles
tcolors = {'r', 'b'};
figure(10); clf; 
set(10, 'Position', [0 0 500 500]); movegui(10, 'center');
hold on;
if paintpatches,
    hpatch = zeros(length(xpatch)-1, length(ypatch) - 1);   % handles
    for i = 1:length(xpatch)-1,
        for j = 1:length(ypatch)-1,
             hpatch(i, j) = patch(xpatch([i  i+1 i+1   i]), ...
                ypatch([j  j   j+1 j+1]), rand * ones(1, 3));
        end;
    end;
else
    x = [1; 2];
    plot(x(1), x(2), 'ko', 'LineWidth', 2);       
    ii = zeros(tdim.c, tdim.cdim);
    for di = 1:tdim.cdim,        
        ii(:, di) = sum(xdata{di} <= x(di), 2);
    end;
    for i = 1:c,
        i1 = ii(i, 1); i2 = ii(i, 2);
        patch(xdata{1}(i, [i1 i1+1 i1+1 i1]), xdata{2}(i, [i2 i2 i2+1 i2+1]), ...
            tcolors{i}, 'FaceAlpha', 0.5);
    end;
end;
axis([min(xpatch) max(xpatch) min(ypatch) max(ypatch)]); hold on;

z = zeros(size(xdata{1}, 2), size(xdata{2}, 2));
for i = 1:c,
    mesh(xdata{1}(i, :), xdata{2}(i, :), z, ...
        'EdgeColor', tcolors{i}, 'FaceColor', 'none', 'LineWidth', 2);
end;

if ~paintpatches, return; end;


% ----------------------------------
% Compute activation matrix and its rank

% Compute and output some dimensions
N = mu * c;
dimp = [(length(xpatch) - 1)  (length(ypatch) - 1)];
Np = prod(dimp);        % number of patches

disp(['Number of patches: ' num2str(prod(dimp)) ' = ' num2str(dimp(1)) 'x' num2str(dimp(2))]);
disp('Number of parameters:');
if cfg.exact,
    % one col & column of parameters are dummies in the tiling that falls
    % precisely on the interval
    mu2 = (length(grids{1})-1) * (length(grids{2})-1); 
    n = mu2 + mu * (c-1)        
else
    n = mu * c
end;

% pick a point in every patch (the center)
xp = xpatch(1:end-1) + diff(xpatch)/2;
yp = ypatch(1:end-1) + diff(ypatch)/2;

% plot the points
[xx, yy] = ndgrid(xp, yp); plot(xx(:), yy(:), 'r.');       

% Compute activation levels
% The activation matrix has an activation vector on each column; 
% and as many columns as there are patches
% Ap = zeros(N, Np);
Ap = zeros(n, Np);      % NOTE problems may arise when n ~= N!

% tiling should vary slowest - permute activation vectors s.t. this is true
tsizeclast = [tdim.tsize(2:end) c];     
permclast = [2 3 1];                    % note hardcoded for 2-dim domain
for i = 1:dimp(1),
    for j = 1:dimp(2),
        [v ti tj] = tiling_get(xdata, wdata, tdim, [xp(i) yp(j)]);
        % reshuffle indices such that tiling varies slowest
        ti = lin2ndi(ti, tdim.tsize);
        ti = ndi2lin(ti(:, permclast), tsizeclast);
        Ap(ti, ndi2lin([i j], dimp)) = 1/c;
    end;
end;

% Check rank of activation matrix
disp('Rank of activation matrix:');
r = rank(Ap)

if r == n, return;         % rank is ok - this will never happen
else
    disp('Rank insufficient, attempting to increase rank');
end;

return

% ----------------------------------
% Attempt to obtain rank n

if join,
    
    tb = 1:mu:n+1;        % tiling borders in an activation vector
    drank = n - r;
    Afr = Ap;
    rankincreased = 1;    % initial value
    
    % join tiles in tilings until full column rank
    % or all tilings have degenerated to one tile
    while (drank > 0) && (size(Afr, 1) > c) && rankincreased,
        rankincreased = 0;
        for tj = 1:c,       % tiling
            for ti = tb(tj) : tb(tj+1)-2,      % all tile indices in tiling tj
%                 disp(['Trying to join tiles ' num2str((ti:ti+1) - tb(tj) + 1) ' in tiling ' num2str(tj)]);
                % try joining the two tiles starting at "ti"
                A2 = [Afr(1:ti-1, :); sum(Afr(ti:ti+1, :)); Afr(ti+2:end, :)];
                dr = size(A2, 1) - rank(A2);
                if dr < drank,
                    dr = drank;
                    % place normalized A2 into Afr
                    Afr = A2; Afr(A2 > 0) = 1/c;
                    tb(tj+1:end) = tb(tj+1:end) - 1;
                    rankincreased = 1;
                    break;
                end;
            end;    % for tile
            if rankincreased, break; end;   % restart cycle
        end;        % for tiling
    end;            % while not fullrank
    if drank > 0,
        disp('Could not decrease rank by joining tiles. Giving up.');
        return;
    end;
else
    deactivated = [];       % remember which patches are deactivated

    % try deactivating patches to get higher rank
    Afr = Ap;       % full rank matrix - hopefully
    while r < n,
        fired = lin2ndi(find(Afr), size(Afr));
        success = 0;
        for k = 1:size(fired, 1),
            % deactivate patch, see if rank increases
            oldcol = Afr(:, fired(k, 2));
            col = oldcol;
            % don't try to deactivate the last active patch, 
            % it doesn't make sense
            if col(fired(k, 1)) == 1, continue; end;
            col(fired(k, 1)) = 0;
            % place renormalized activation vector into matrix
            Afr(:, fired(k, 2)) = col / sum(col);
            rr = rank(Afr);
            if rr > r, 
                deactivated = [deactivated; fired(k, :)];
                success = 1;
                r = rr;
                break;
            else
                % restore matrix to state prior to deactivation
                Afr(:, fired(k, 2)) = oldcol;
            end;
        end;
        if ~success,
            disp('Could not decrease rank by single deactivations. Giving up.');
            return;
        end;
    end;
end;

% ----------------------------------
% Find a full-rank square matrix within Afr

disp('Reordering to get basis on first rows');
bi = ones(Np, 1);
k = 1;
while (sum(bi) > n) && (k <= Np),
    bi(k) = 0;
    % check if element is needed for full-rank matrix
    if rank(Afr(:, find(bi))) < n, bi(k) = 1;
        % else leave it at 0
    end;
    k = k + 1;
end;

if sum(bi) > n,
    disp('Reordering failed, could not find basis');
    return;
end;

% reorder such that the full-rank square matrix is first,
% i.e. such that the first n patches / points form a basis
perm = [find(bi); find(~bi)];
A = Afr(:, perm);

% ----------------------------------
% find out coefficients alpha_i for i = n+1:Np such that 
% the activation vector of patch i is a linear combination of the
% activations of the basis patches 1:n with coefficients alpha

% activation vectors are now on rows, so we need to transpose
alpha = inv(A(:, 1:n)) * A(:, n+1:end);

if any(sum(abs(alpha)) > 1),
    disp('Absolute values of weights in linear combination');
    disp('    of basis vectors exceed 1 for at least one patch');
end;
    